INTERNAL_TO_DECISION = {
    "COST_WARN_INCREASE": "COST_INCREASE_WARN",
    "COST_BLOCK_INCREASE": "COST_INCREASE_BLOCK",
    "LATENCY_WARN_INCREASE": "LATENCY_INCREASE_WARN",
    "LATENCY_BLOCK_INCREASE": "LATENCY_INCREASE_BLOCK",
    "PII_BLOCK_EMAIL": "PII_EMAIL_BLOCK",
    "PII_BLOCK_PHONE": "PII_PHONE_BLOCK",
    "PII_BLOCK_CREDIT_CARD": "PII_CREDIT_CARD_BLOCK",
    "PII_BLOCK_SSN": "PII_SSN_BLOCK",
    "DRIFT_BLOCK_EMPTY": "DRIFT_EMPTY_OUTPUT_BLOCK",
    "DRIFT_WARN_SHORT_OUTPUT": "DRIFT_TOO_SHORT_WARN",
    "DRIFT_WARN_EXPANSION": "DRIFT_EXPANSION_WARN",
    "DRIFT_BLOCK_EXPANSION": "DRIFT_EXPANSION_BLOCK",
    "DRIFT_WARN_COMPRESSION": "DRIFT_COMPRESSION_WARN",
    "DRIFT_BLOCK_COMPRESSION": "DRIFT_COMPRESSION_BLOCK",
    "DRIFT_WARN_LOW_SIMILARITY": "DRIFT_SIMILARITY_WARN",
    "RED_TEAM_BLOCK_INJECTION": "RED_TEAM_INJECTION_BLOCK",
    "RED_TEAM_BLOCK_TOXICITY": "RED_TEAM_TOXICITY_BLOCK",
    "RED_TEAM_BLOCK_COMPETITORS": "RED_TEAM_COMPETITORS_BLOCK",
    "CONTRACT_BLOCK_INVALID_JSON": "OUTPUT_CONTRACT_INVALID_JSON_BLOCK",
    "CONTRACT_WARN_INVALID_JSON": "OUTPUT_CONTRACT_INVALID_JSON_WARN",
    "CONTRACT_BLOCK_TYPE_CHANGE": "OUTPUT_CONTRACT_TYPE_CHANGE_BLOCK",
    "CONTRACT_WARN_MISSING_KEYS": "OUTPUT_CONTRACT_MISSING_KEYS_WARN",
    "CONTRACT_WARN_TYPE_MISMATCH": "OUTPUT_CONTRACT_TYPE_MISMATCH_WARN",
    "STRICT_PROMOTED_WARN": "STRICT_MODE_PROMOTION_BLOCK",
}

DECISION_TO_INTERNAL = {v: k for k, v in INTERNAL_TO_DECISION.items()}
